//
//  QuizPlayViewController.swift
//  okgo
//
//  Created by Raj Dhakate on 19/01/21.
//

import UIKit
import AVKit

class QuizPlayViewController: UIViewController {
    
    private var streamController = StreamViewController()
    
    //MARK: View Controller Life Cycle
    deinit {
        print("Dealloc: " + String(describing: self))
        UIApplication.shared.isIdleTimerDisabled = false
        NotificationCenter.default.removeObserver(self)
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        UIApplication.shared.isIdleTimerDisabled = true
        
        setupViews()
    }
    
    override func viewDidLayoutSubviews() {
        super.viewDidLayoutSubviews()
    }
    
    @objc private func closeViewController(_ dismissToHome: Bool = false) {
        self.streamController.closeViewController()
        self.dismiss(animated: true, completion: nil)
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
    }
    
    //MARK: Setup Views
    private func setupViews() {
        
        let back = UIBarButtonItem(image: UIImage(named: "cancel")?.withRenderingMode(.alwaysTemplate), style: .plain, target: self, action: #selector(closeViewController))
        back.tintColor = UIColor.white
        self.navigationItem.rightBarButtonItem = back
        
        streamController.delegate = self
        
        Functions.addConstraintsAndInsertSubview(addView: streamController.view, onView: self.view, atIndex: 0)
        
        // other views
        
        // .. start the stream
        startStream()
    }
    
    //MARK: Start Stream
    private func startStream() {
        // start the stream
        streamController.start()
    }
    
    //MARK: Stop Stream
    private func stopStream() {
        // stop the stream
        streamController.stop()
    }
}


class Functions {
    class func addConstraintsAndInsertSubview(addView: UIView, onView: UIView, atIndex: Int) {
        onView.insertSubview(addView, at: atIndex)
        addView.translatesAutoresizingMaskIntoConstraints = false
        addView.topAnchor.constraint(equalTo: onView.topAnchor, constant: 0).isActive = true
        addView.bottomAnchor.constraint(equalTo: onView.bottomAnchor, constant: 0).isActive = true
        addView.leadingAnchor.constraint(equalTo: onView.leadingAnchor, constant: 0).isActive = true
        addView.trailingAnchor.constraint(equalTo: onView.trailingAnchor, constant: 0).isActive = true
    }
}
