//
//  StreamViewController.swift
//  okgo
//
//  Created by Raj Dhakate on 09/02/21.
//

import UIKit
import R5Streaming

class StreamViewController: R5VideoViewController {
    
    // config based variables
    var streamKey: String? = nil
    var host: String? = nil
    var port: Int32? = nil
    var context: String? = nil
    var RED5_SERVER_SDK_KEY: String? = nil
    
    weak var delegate: QuizPlayViewController?
    
    private var config: R5Configuration?
    private var stream: R5Stream?
    private var connection: R5Connection?
    
    private var isResumable: Bool = false
    private var isStreamValid: Bool = false
    private var sharedObject: R5SharedObject?
    private var sharedObjectConfig: R5Configuration?
    private var sharedObjectConnection: R5Connection?
    private var isSharedObjectConnected : Bool = false
    private var timer = Timer()
    
    deinit {
        print("Dealloc: " + String(describing: self))
        NotificationCenter.default.removeObserver(self)
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.view.backgroundColor = .black
        
        NotificationCenter.default.addObserver(self, selector: #selector(applicationDidEnterBackgroundNotification(notification:)), name: UIApplication.didEnterBackgroundNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(applicationWillResignActiveNotification(notification:)), name: UIApplication.willResignActiveNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(applicationDidBecomeActiveNotification(notification:)), name: UIApplication.didBecomeActiveNotification, object: nil)
        
        streamKey = <#streamKey#>
        context = "live"
        port = 8554
        host = <#host#>
        RED5_SERVER_SDK_KEY = <#SDK_KEY#>
        
        // set config
        setConfigs()
        
        // start shared object stream
        startShareObjectStream()
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
        if let streamKey = streamKey, isStreamValid, isResumable {
            isResumable = false
            stream?.play(streamKey)
        }
    }

    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)

        if stream?.mode() == r5_stream_mode_streaming, !isResumable {
            isResumable = true
            stream?.stop()
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        if let streamKey = streamKey, isStreamValid, isResumable {
            isResumable = false
            stream?.play(streamKey)
        }
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        
        if stream?.mode() == r5_stream_mode_streaming, !isResumable {
            isResumable = true
            stream?.stop()
        }
    }
    
    func closeViewController() {
        stop()
        closeSObject()
    }
    
    private func setConfigs() {
        // Set up the configuration
        
        // create streaming config
        config = R5Configuration()
        config?.host = host
        config?.port = port ?? 0
        config?.contextName = context
        config?.streamName = streamKey
        config?.licenseKey = RED5_SERVER_SDK_KEY
        
        // create shared object config
        sharedObjectConfig = R5Configuration()
        sharedObjectConfig?.host = host
        sharedObjectConfig?.port = port ?? 0
        sharedObjectConfig?.contextName = context
        sharedObjectConfig?.streamName = (streamKey ?? "") + "_chat"
        sharedObjectConfig?.licenseKey = RED5_SERVER_SDK_KEY
        
        isStreamValid = true
    }
    
    func start() {
        connection = R5Connection(config: config)
        connection?.delegate = self
        connection?.client = self
        
        stream = R5Stream(connection: connection)
        stream?.delegate = self
        
        self.attach(stream)
        
        if let streamKey = streamKey, isStreamValid {
            stream?.play(streamKey)
        }
    }
    
    func stop() {
        if let stream = stream {
            stream.stop()
            stream.delegate = nil
        }
        
        if let connection = connection {
            connection.delegate = nil
            connection.client = nil
        }
        
        stream = nil
        config = nil
        connection = nil
    }
    
    private func closeSObject() {
        timer.invalidate()
        
        sharedObject?.close()
        sharedObject?.client = nil
        
        sharedObjectConnection?.delegate = nil
        sharedObjectConnection?.client = nil
        
        sharedObject = nil
        sharedObjectConfig = nil
        sharedObjectConnection = nil
    }
    
    //MARK: Handle Chat
    func addMessage(message: [String: Any], update: Bool = true) {
        
        DispatchQueue.main.async { [weak self] in
            guard let self = self else { return }
            
            if let delegate = self.delegate, update {
                
            }
            
        }
    }
    
    //callback for remote object connection - remote object now available
    @objc func onSharedObjectConnect(objectValue: NSDictionary) {
        isSharedObjectConnected = true
        if let data : NSMutableDictionary = (sharedObject?.data) {
            NSLog("R5Connection : Shared Object Listener : \n")
            NSLog("R5Connection : Shared Object \(data)")
            
            if let messageIn = data as? [String: Any], !messageIn.isEmpty {
                messageTransmit(messageIn: messageIn)
            }
        }
    }

    @objc func sendMessage(message: String) {
        let messageOut : [AnyHashable:Any] = ["user": "Test", "message": message]
        
        //Calls for the relevant method with the sent parameters on all clients listening to the shared object
        //Note - This includes the client that sends the call
        sharedObject?.send("messageTransmit", withParams: messageOut)
        
        // update message view to nil
        if let delegate = delegate {
            
        }
    }
    
    @objc func messageTransmit(messageIn: [String: Any]) {
        addMessage(message: messageIn, update: true)
    }
    
    func startShareObjectStream() {
        // Set up the connection and stream
        sharedObjectConnection = R5Connection(config: sharedObjectConfig)
        sharedObjectConnection?.delegate = self
        sharedObjectConnection?.client = self
        
        if isStreamValid {
            sharedObjectConnection?.startDataOnlyStream()
        }
    }
}

extension StreamViewController: R5StreamDelegate {
    
    func onR5StreamStatus(_ stream: R5Stream!, withStatus statusCode: Int32, withMessage msg: String!) {
        NSLog("R5Connection : R5Stream Status: %s ", r5_string_for_status(statusCode))
    }
    
}

extension StreamViewController: R5ConnectionDelegate {
    func onR5ConnectionStatus(_ connection: R5Connection!, withStatus statusCode: Int32, withMessage msg: String!) {
        NSLog("R5Connection : Status: %s ", r5_string_for_status(statusCode))
        
        if (Int(statusCode) == Int(r5_status_start_streaming.rawValue)) {
            self.timer = Timer.scheduledTimer(timeInterval: 0.25, target: self, selector: #selector(SOConnect), userInfo: nil, repeats: false)
        }
    }
    
    @objc func SOConnect() {
        NSLog("R5Connection : Sending shared object connection request")
        
        if let sharedObjectConfig = sharedObjectConfig, let streamName = sharedObjectConfig.streamName {
            sharedObject = R5SharedObject(name: streamName + "_chat", connection: sharedObjectConnection)
            sharedObject?.client = self
        }
        
    }
}

extension StreamViewController {
    @objc func applicationDidEnterBackgroundNotification(notification: Notification) {
        
        if stream?.mode() == r5_stream_mode_streaming, !isResumable {
            isResumable = true
            stream?.stop()
        }
        
    }
    
    @objc func applicationWillResignActiveNotification(notification: Notification) {
        
        if stream?.mode() == r5_stream_mode_streaming, !isResumable {
            isResumable = true
            stream?.stop()
        }
        
    }
    
    @objc func applicationDidBecomeActiveNotification(notification: Notification) {
        
        if let streamKey = streamKey, isStreamValid, isResumable {
            isResumable = false
            stream?.play(streamKey)
        }
        
    }
}
